#! /usr/bin/env python
# -*- coding: UTF-8 -*-

from edt2ics.ical import iCalSchedule
from edt2ics.scraper import ScheduleScraper

__version__ = '0.1.1'

# silent pyflakes
iCalSchedule, ScheduleScraper



#! /usr/bin/env python
# -*- coding: UTF-8 -*-

from datetime import date, datetime, timedelta
from icalendar import Calendar, Event, vRecur
import json
import os.path
from os.path import dirname
from uuid import uuid4


class iCalSchedule(object):

    DAYS = ['MO', 'TU', 'WE', 'TH', 'FR', 'SA', 'SU']

    def __init__(self, scraper=None, startdate=None, enddate=None):
        if startdate is None or enddate is None:
            dts, dte = self._get_dates(scraper.period, scraper.year,
                    scraper.semester)

            if startdate is None:
                startdate = dts
            if enddate is None:
                enddate = dte

        self.startdate = startdate
        self.enddate = enddate
        self._first_weekdays = {} # cache
        self._init_ical()
        if scraper is not None:
            for ev in scraper.get_events():
                self.add_event(ev)


    def _init_ical(self):
        cal = Calendar()
        cal.add('version', '2.0')
        cal.add('prodid', '-//edt2ics//bfontaine.net//')
        cal.add('method', 'publish')
        self.cal = cal


    def _recur_params(self, wday):
        return {
            'freq': 'weekly',
            'wkst': self.DAYS[0],
            'byday': self.DAYS[wday],
            'until': self.enddate,
        }


    def _get_first_weekday(self, day):
        """
        Return the first date after ``self.startdate`` which is on the given
        weekday (0=Monday, 1=Tuesday, etc)
        """
        if day not in self._first_weekdays:
            start_wd = self.startdate.weekday()
            delta = (day - start_wd + 7) % 7
            self._first_weekdays[day] = self.startdate + timedelta(days=delta)
        return self._first_weekdays[day]


    def _get_dates(self, period, year, semester):
        source = os.path.join(dirname(__file__), 'dates.json')
        with open(source, 'r') as f:
            data = json.loads(f.read())
        dates = data['dates'][period][str(semester)][year]
        start, end = dates['start'], dates['end']

        return self._str2date(start), self._str2date(end)


    def _str2date(self, s):
        return date(*map(lambda e: int(e, 10), s.split('-')))


    def add_event(self, ev):
        """
        Add a new recurrent event to this schedule
        """
        day = self._get_first_weekday(ev.day)
        dtstart = datetime.combine(day, ev.tstart)
        dtend = datetime.combine(day, ev.tend)

        iev = Event()
        iev.add('uid', str(uuid4()))
        iev.add('status', 'confirmed')
        iev.add('dtstart', dtstart)
        iev.add('dtend', dtend)
        iev.add('rrule', vRecur(self._recur_params(ev.day)))
        iev.add('summary', '%s %s' % (ev.type_, ev.title))
        iev.add('location', ev.room)
        iev.add('description', ev.prof)
        self.cal.add_component(iev)


    def to_ical(self):
        return self.cal.to_ical()



#! /usr/bin/env python
# -*- coding: UTF-8 -*-


class ScheduleEvent(object):
    """
    A schedule event
    """

    def __init__(self, **kwargs):
        """
        Attributes: color, title, type_, day, tstart, tend, duration, room,
        prof.
        """
        for k, v in kwargs.items():
            setattr(self, k, v)



#! /usr/bin/env python
# -*- coding: UTF-8 -*-

"""
This module provides a class to parse schedule events from Paris Diderot's
Computer Science departement website.
"""

from bs4 import BeautifulSoup
from datetime import date, time, timedelta
import re

try:
    from urllib2 import urlopen
    from urlparse import urljoin
except ImportError:
    from urllib.request import urlopen
    from urllib.parse import urljoin


from edt2ics.events import ScheduleEvent

HOST = 'magma.informatique.univ-paris-diderot.fr:2201'
PATH_FMT = '/~ufr/UFR{period}/EDT/visualiserEmploiDuTemps.php?' \
           'quoi={year},{semester}'

RE_DESC = re.compile(
    r"""
    (?P<type>[-\w]+)\s+             # COURS, TP_M2, etc
    (?P<title>.+)\s+:\s+            # course name :
    (?P<day>[a-z]+di)\s+            # {lun,mar,...}di (day)
    (?P<time>\d{1,2}h(?:\d{2})?)\s+ # 12h30
    \(dur.e\s+:\s+                  # (durée :
    (?P<duration>\dh(?:\d{2})?)\)   #  2h30)
    """,
    re.VERBOSE)
RE_ROOM = re.compile(r'^\d') # detect a room number
RE_YEAR_SUFFIX = re.compile(r'_[ML][1-3]$')

class ScheduleScraper(object):
    """
    A scraper for an HTML schedule
    """

    DAYS = ['lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi']


    def __init__(self, year, semester=1, period=None, host=None, **kwargs):
        """
        - year: L1, L2, L3, M1 or M2
        - semester: 1 or 2
        - period: '2013-2014', '2014-2015', etc
        """
        if period is None:
            period = self._guess_period()

        if host is None:
            host = HOST

        path = PATH_FMT.format(year=year, semester=semester, period=period)
        self.url = urljoin('http://%s' % host, path)
        self._fetch()
        self.year = year
        self.semester = semester
        self.period = period


    def _fetch(self):
        """
        Fetch the remote page
        """
        stream = urlopen(self.url)
        self._soup = BeautifulSoup(stream, 'lxml')


    def _guess_period(self):
        """
        Guess a period (e.g. '2014-2015') from the current date
        """
        today = date.today()
        year = today.year
        if today.month < 7:
            year -= 1
        return '%04d-%04d' % (year, year+1)


    def get_events(self):
        """
        Yield each event from the parsed page
        """
        for td in self._soup.select('td[title]'):
            desc = td.attrs.get('title')

            profs = td.select('small')
            prof = profs[0].get_text().strip() if profs else None

            for s in td.strings:
                s = s.strip()
                if RE_ROOM.match(s):
                    room = s
                    break
            else:
                room = None

            info = RE_DESC.match(desc)
            if not info:
                yield None
                continue

            tstart = self.parse_time(info.group('time'))
            h, m = self._parse_hm(info.group('duration'))
            endminutes = (tstart.hour + h) * 60 + tstart.minute + m
            tend = time(endminutes / 60, endminutes % 60)

            kw = {
                'color': td.attrs.get('bgcolor'),
                'title': info.group('title'),
                'type_': self.parse_type(info.group('type')),
                'day': self.parse_day(info.group('day')),
                'tstart': tstart,
                'tend': tend,
                'duration': timedelta(hours=h, minutes=m),
                'room': room,
                'prof': prof,
            }

            yield ScheduleEvent(**kw)


    def parse_day(self, title):
        """
        Parse a day from the page and return an integer (0 for Monday, 1 for
        Tuesday and so on).
        """
        if not title or not title in self.DAYS:
            return None
        return self.DAYS.index(title)


    def _parse_hm(self, text):
        """
        Parse a time given as '<hh>h[<mm>]' and return a tuple
        """
        t = text.split('h')
        h = int(t[0])
        m = int(t[1] or '0')
        return h, m


    def parse_time(self, title):
        """
        Parse a time and return a ``time`` object.
        """
        h, m = self._parse_hm(title)
        return time(h, m)


    def parse_type(self, type_):
        """
        Parse an event type
        """
        type_ = re.sub(RE_YEAR_SUFFIX, '', type_.strip())
        if len(type_) == 2:
            return type_.upper()
        return type_.capitalize()



#! /usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import print_function

import argparse
import sys

from edt2ics import ScheduleScraper, iCalSchedule


def write_ical(ical, output):
    if output == '-':
        sys.stdout.write(ical)
    else:
        with open(output, 'w') as f:
            f.write(ical)


def main():
    parser = argparse.ArgumentParser(description='P7 iCalendar schedule converter')
    parser.add_argument('year', type=str, help='L3, M1, or M2')
    parser.add_argument('--semester', dest='semester', type=int, default=1,
            help='semester')
    parser.add_argument('--host', dest='host', type=str, default=None,
            help='remote host')
    parser.add_argument('--output', dest='output', type=str, default=None,
            help='output file')
    args = parser.parse_args()

    try:
        s = ScheduleScraper(**vars(args))
        ics = iCalSchedule(s)

        output = args.output if args.output else '%s.ics' % args.year
        write_ical(ics.to_ical(), output)
        print('--> %s' % output)
    except KeyboardInterrupt:
        pass
